==> jquery-packir-0.1.1.tgz: package/lib/packir.js <==
var inherits          = require('inherits-js'),
    packir            = require("packir"),
    BaseStrategy      = packir.BaseStrategy,
    StackCompressor   = packir.StackCompressor,
    Rule              = packir.Rule,
    NonrecurringXRule = packir.NonrecurringXRule,
    NonrecurringYRule = packir.NonrecurringYRule,
    RecurringXRule    = packir.RecurringXRule,
    Area              = packir.Area,
    _                 = require('lodash');

var DATA_INSTANCE = "packir_instance";

/**
 * @class Packir
 * @constructor
 * @abstract
 *
 * @param {DOMElement} [$el]
 * @param {Object} [opts]
 */
function Packir($el, opts) {
    var options, strategy, compressor;

    this.$el = $el;
    this.options = options = _.extend({}, this.constructor.DEFAULTS, opts);

    this.state = {
        active: false
    };

    switch (options.strategy) {
        default: {
            strategy = new BaseStrategy();
        }
    }

    switch (options.compressor) {
        default: {
            compressor = new StackCompressor();
        }
    }

    strategy.injectCompressor(compressor);

    _.forEach(options.rules, function(def) {
        var rule, options, r;

        if (_.isArray(def)) {
            rule = def[0];
            options = def[1];
        } else {
            rule = def;
        }

        switch (rule) {
            case "nonrecurring-y": {
                r = new NonrecurringYRule(options);
                break;
            }

            case "nonrecurring-x": {
                r = new NonrecurringXRule(options);
                break;
            }

            case "recurring-x": {
                r = new RecurringXRule(options);
                break;
            }

            default: {
                if (rule instanceof Rule) {
                    return rule;
                }

                throw new Error('Unknown rule: "' + rule + '"');
            }
        }

        strategy.addRule(r);
    });

    this.strategy = strategy;

    this.children = $el.children();
    this.blocks = $el.children().map(function(index) {
        var $clone;

        $clone = $(this).clone(true, true);

        return {
            width:  $clone.data(options.data.width)  || $clone.width(),
            height: $clone.data(options.data.height) || $clone.height(),
            weight: $clone.data(options.data.weight) || index,
            $el:    $clone
        };
    });

    $el.data(DATA_INSTANCE, this);
}

Packir.prototype = {
    constructor: Packir,

    run: function(done) {
        var self = this,
            $el, area, options;

        if (this.state.active) {
            this.reset();
        }

        options = this.options;

        $el = this.$el;

        area = new Area($el.width(), $el.height());
        area.injectStrategy(this.strategy);

        area.fill(this.blocks, options.options, function(err, grid) {
            var $grid;

            if (err) {
                done(err);
                return;
            }

            function X(v) {
                return v * grid.stepLeft;
            }

            function Y(v) {
                return v * grid.stepTop;
            }

            $grid = $("<div/>")
                .css({
                    width: X(grid.width) + 'px',
                    height: Y(grid.height) + 'px'
                })
                .addClass(options.className.grid);

            _.forEach(grid.rows, function(row, index, list) {
                var $row, lastRow, firstRow;

                firstRow = index == 0;
                lastRow = index == (list.length - 1);

                $row = $("<div/>")
                    .css({
                        width: X(row.height) + 'px',
                        height: Y(row.width) + 'px',
                        top: Y(row.position.top) + 'px',
                        left: X(row.position.left) + 'px',
                        position: "absolute"
                    })
                    .addClass(options.className.row);

                if (firstRow) {
                    $row.addClass(options.className.rowTop);
                }

                if (lastRow) {
                    $row.addClass(options.className.rowBottom);
                }

                _.forEach(row.stacks, function(stack, index, list) {
                    var $stack, firstStack, lastStack;

                    firstStack = index == 0;
                    lastStack = index == (list.length - 1);

                    $stack = $("<div/>")
                        .css({
                            position: "absolute",
                            top: Y(stack.position.top) + 'px',
                            left: X(stack.position.left) + 'px',
                            width: X(stack.width) + 'px',
                            height: Y(stack.height) + 'px'
                        })
                        .addClass(options.className.stack);

                    if (firstStack) {
                        $stack.addClass(options.className.stackLeft);
                    }

                    if (lastStack) {
                        $stack.addClass(options.className.stackRight);
                    }

                    _.forEach(stack.fields, function(field) {
                        var $field, $el, style;

                        $el = field.data.$el;

                        $field = $("<div/>").addClass(options.className.field);
                        style = {
                            position: "absolute",
                            left: X(field.position.left) + 'px',
                            top: Y(field.position.top) + 'px',
                            width: X(field.width) + 'px',
                            height: Y(field.height) + 'px',
                            overflow: 'hidden'
                        };

                        if (firstRow) {
                            $field.addClass(options.className.fieldTop);

                            if (options.className.elTop) {
                                $el.addClass(options.className.elTop);
                            }
                        }

                        if (lastRow) {
                            $field.addClass(options.className.fieldBottom);

                            if (options.className.elBottom) {
                                $el.addClass(options.className.elBottom);
                            }
                        }

                        if (firstStack) {
                            $field.addClass(options.className.fieldLeft);

                            if (options.className.elLeft) {
                                $el.addClass(options.className.elLeft);
                            }
                        }

                        if (lastStack) {
                            $field.addClass(options.className.fieldRight);

                            if (options.className.elRight) {
                                $el.addClass(options.className.elRight);
                            }
                        }

                        $field.css(style);

                        $field.append($el);
                        $stack.append($field);
                    });

                    $row.append($stack);
                });

                $grid.append($row);
            });

            self.children = _.map(self.children, function(el) {
                return $(el).detach();
            });

            self.setState({
                active: true
            });

            $el.append($grid);

            self.$grid = $grid;

            done();
        });
    },

    reset: function() {
        var self = this;

        _.forEach(this.children, function($el) {
            $el.appendTo(self.$el);
        });

        this.$grid.remove();

        this.setState({ active: false });
    },

    setState: function(state) {
        _.extend(this.state, state);
    },

    method: function(name, options, cb) {
        switch (name) {
            case "reset": {
                this.reset();
                return cb();
            }

            default: {
                throw new TypeError("Unknown method '" + name + "'");
            }
        }
    }
};

Packir.DEFAULTS = {
    strategy: "base",
    compressor: "stack",
    rules: [
        "nonrecurring",
        "recurring"
    ],
    data: {
        width:  "width",
        height: "height",
        weight: "weight"
    },
    options: {
        margin: {
            x: 0,
            y: 0
        }
    },
    className: {
        grid:        "packir-grid",
        row:         "packir-row",
        rowTop:      "packir-row--top",
        rowBottom:   "packir-row--bottom",
        stack:       "packir-stack",
        stackLeft:   "packir-stack--left",
        stackRight:  "packir-stack--right",
        field:       "packir-field",
        fieldTop:    "packir-field--top",
        fieldRight:  "packir-field--right",
        fieldBottom: "packir-field--bottom",
        fieldLeft:   "packir-field--left",
        elTop:       null,
        elRight:     null,
        elBottom:    null,
        elLeft:      null
    }
};

Packir.find = function($el) {
    return $el.data(DATA_INSTANCE);
};

Packir.extend = function(proto, statics) {
    return inherits(this, proto, statics);
};

module.exports = Packir;

==> jquery-packir-0.1.9.tgz: package/lib/packir.js <==
var inherits          = require('inherits-js'),
    packir            = require("packir"),
    BaseStrategy      = packir.BaseStrategy,
    StackCompressor   = packir.StackCompressor,
    Rule              = packir.Rule,
    NonrecurringXRule = packir.NonrecurringXRule,
    NonrecurringYRule = packir.NonrecurringYRule,
    RecurringXRule    = packir.RecurringXRule,
    Area              = packir.Area,
    _                 = require('lodash');

var DATA_INSTANCE = "packir_instance";

/**
 * @class Packir
 * @constructor
 * @abstract
 *
 * @param {DOMElement} [$el]
 * @param {Object} [opts]
 */
function Packir($el, opts) {
    var options, strategy, compressor;

    this.$el = $el;
    this.options = options = _.extend({}, this.constructor.DEFAULTS, opts);

    this.state = {
        active: false
    };

    switch (options.strategy) {
        default: {
            strategy = new BaseStrategy();
        }
    }

    switch (options.compressor) {
        default: {
            compressor = new StackCompressor();
        }
    }

    strategy.injectCompressor(compressor);

    _.forEach(options.rules, function(def) {
        var rule, options, r;

        if (_.isArray(def)) {
            rule = def[0];
            options = def[1];
        } else {
            rule = def;
        }

        switch (rule) {
            case "nonrecurring-y": {
                r = new NonrecurringYRule(options);
                break;
            }

            case "nonrecurring-x": {
                r = new NonrecurringXRule(options);
                break;
            }

            case "recurring-x": {
                r = new RecurringXRule(options);
                break;
            }

            default: {
                if (rule instanceof Rule) {
                    return rule;
                }

                throw new Error('Unknown rule: "' + rule + '"');
            }
        }

        strategy.addRule(r);
    });

    this.strategy = strategy;

    this.children = $el.children();
    this.blocks = $el.children().map(function(index) {
        var $clone;

        $clone = $(this).clone(true, true);

        return {
            width:  $clone.data(options.data.width)  || $clone.width(),
            height: $clone.data(options.data.height) || $clone.height(),
            weight: $clone.data(options.data.weight) || index,
            $el:    $clone
        };
    });

    $el.data(DATA_INSTANCE, this);
}

Packir.prototype = {
    constructor: Packir,

    run: function(done) {
        var self = this,
            $el, area, options;

        if (this.state.active) {
            this.reset();
        }

        options = this.options;

        $el = this.$el;

        area = new Area($el.width(), $el.height());
        area.injectStrategy(this.strategy);

        area.fill(this.blocks, options.options, function(err, grid) {
            var $grid;

            if (err) {
                done(err);
                return;
            }

            function X(v) {
                return v * grid.stepLeft;
            }

            function Y(v) {
                return v * grid.stepTop;
            }

            $grid = $("<div/>")
                .css({
                    width: X(grid.width) + 'px',
                    height: Y(grid.height) + 'px'
                })
                .addClass(options.className.grid);

            _.forEach(grid.rows, function(row, index, list) {
                var $row, lastRow, firstRow;

                firstRow = index == 0;
                lastRow = index == (list.length - 1);

                $row = $("<div/>")
                    .css({
                        width: X(row.width) + 'px',
                        height: Y(row.height) + 'px',
                        top: Y(row.position.top) + 'px',
                        left: X(row.position.left) + 'px',
                        position: "absolute"
                    })
                    .addClass(options.className.row);

                if (firstRow) {
                    $row.addClass(options.className.rowTop);
                }

                if (lastRow) {
                    $row.addClass(options.className.rowBottom);
                }

                _.forEach(row.stacks, function(stack, index, list) {
                    var $stack, firstStack, lastStack;

                    firstStack = index == 0;
                    lastStack = index == (list.length - 1);

                    $stack = $("<div/>")
                        .css({
                            position: "absolute",
                            top: Y(stack.position.top) + 'px',
                            left: X(stack.position.left) + 'px',
                            width: X(stack.width) + 'px',
                            height: Y(stack.height) + 'px'
                        })
                        .addClass(options.className.stack);

                    if (firstStack) {
                        $stack.addClass(options.className.stackLeft);
                    }

                    if (lastStack) {
                        $stack.addClass(options.className.stackRight);
                    }

                    _.forEach(stack.fields, function(field) {
                        var $field, $el, style;

                        $el = field.data.$el;

                        $field = $("<div/>").addClass(options.className.field);
                        style = {
                            position: "absolute",
                            left: X(field.position.left) + 'px',
                            top: Y(field.position.top) + 'px',
                            width: X(field.width) + 'px',
                            height: Y(field.height) + 'px',
                            overflow: 'hidden'
                        };

                        if (firstRow) {
                            $field.addClass(options.className.fieldTop);

                            if (options.className.elTop) {
                                $el.addClass(options.className.elTop);
                            }
                        }

                        if (lastRow) {
                            $field.addClass(options.className.fieldBottom);

                            if (options.className.elBottom) {
                                $el.addClass(options.className.elBottom);
                            }
                        }

                        if (firstStack) {
                            $field.addClass(options.className.fieldLeft);

                            if (options.className.elLeft) {
                                $el.addClass(options.className.elLeft);
                            }
                        }

                        if (lastStack) {
                            $field.addClass(options.className.fieldRight);

                            if (options.className.elRight) {
                                $el.addClass(options.className.elRight);
                            }
                        }

                        $field.css(style);

                        $field.append($el);
                        $stack.append($field);
                    });

                    $row.append($stack);
                });

                $grid.append($row);
            });

            self.children = _.map(self.children, function(el) {
                return $(el).detach();
            });

            self.setState({
                active: true
            });

            $el.append($grid);

            self.$grid = $grid;

            done();
        });
    },

    reset: function() {
        var self = this;

        _.forEach(this.children, function($el) {
            $el.appendTo(self.$el);
        });

        this.$grid.remove();

        this.setState({ active: false });
    },

    setState: function(state) {
        _.extend(this.state, state);
    },

    method: function(name, options, cb) {
        switch (name) {
            case "reset": {
                this.reset();
                return cb();
            }

            default: {
                throw new TypeError("Unknown method '" + name + "'");
            }
        }
    }
};

Packir.DEFAULTS = {
    strategy: "base",
    compressor: "stack",
    rules: [
        "nonrecurring",
        "recurring"
    ],
    data: {
        width:  "width",
        height: "height",
        weight: "weight"
    },
    options: {
        margin: {
            x: 0,
            y: 0
        }
    },
    className: {
        grid:        "packir-grid",
        row:         "packir-row",
        rowTop:      "packir-row--top",
        rowBottom:   "packir-row--bottom",
        stack:       "packir-stack",
        stackLeft:   "packir-stack--left",
        stackRight:  "packir-stack--right",
        field:       "packir-field",
        fieldTop:    "packir-field--top",
        fieldRight:  "packir-field--right",
        fieldBottom: "packir-field--bottom",
        fieldLeft:   "packir-field--left",
        elTop:       null,
        elRight:     null,
        elBottom:    null,
        elLeft:      null
    }
};

Packir.find = function($el) {
    return $el.data(DATA_INSTANCE);
};

Packir.extend = function(proto, statics) {
    return inherits(this, proto, statics);
};

module.exports = Packir;
